(* GNU-GPL
 * System Compression - Logiciel de Compression/Decompression
 * Copyright (c) 2002 Ulrich Van Den Hekke (ulrich.vdh@free.fr)
 * 24 rue Jean Jaures
 * 59560 Comines
 *
 * Ce programme est libre, vous pouvez le redistribuer et/ou le modifier selon
 * les termes de la Licence Publique Gnrale GNU publie par la
 * Free Software Foundation (version 2 ou bien toute autre version ultrieure
 * choisie par vous).
 *
 * Ce programme est distribu car potentiellement utile, mais
 * SANS AUCUNE GARANTIE, ni explicite ni implicite, y compris les garanties de
 * commercialisation ou d'adaptation dans un but spcifique. Reportez-vous  la
 * Licence Publique Gnrale GNU pour plus de dtails.
 *
 * Vous devez avoir reu une copie de la Licence Publique Gnrale GNU en mme
 * temps que ce programme ; si ce n'est pas le cas, crivez  la
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, tats-Unis.
 *
 *)

(******************************************************************************)
(* Type et constante communs au logiciel.                                     *)
(* Version : 1.0.0.0                                                          *)
(* Author : Ulrich Van Den Hekke                                              *)
(* Create day : 06/07/2002                                                    *)
(******************************************************************************)

unit Common;

interface

uses Windows, SysUtils, Classes;

type
  TInfosName    = array [1..16] of char;
  TInfosDesc    = array [1..64] of char;
  TInfosSerial  = array [1..24] of char;
  TInfosVersion = Byte;
  TInformations = record
    iName        : TInfosName;
    iDescription : TInfosDesc;
    iSerial      : TInfosSerial;
    iVersion     : TInfosVersion;
  end;

  TGetByte      = function(var Buffer: Byte): boolean of object; stdcall;
  TPutByte      = procedure(const Buffer: Byte; const flush: boolean) of object; stdcall;

  TInfoProc     = function  (): TInformations; stdcall;
  TAboutProc    = procedure (hWnd: HWND); stdcall;
  TPackProc     = procedure (var BytesWritten: Integer; const getProc: TGetByte; const putProc: TPutByte); stdcall;
  TUnpackProc   = procedure (const TextSize: Integer; const getProc: TGetByte; const putProc: TPutByte); stdcall;

  EPack         = class(Exception);

  (* TRealFileInfo *)
  TRealFileInfo    = packed record
    rfiName,                                        // Nom du fichier                     (*  64b *)
    rfiPath         : array[1..64] of char;         // Nom du dossier                     (*  64b *)
    rfiModifiedDate : TDateTime;                    // Date de derniere modification      (*   8b *)
    rfiSize         : LongInt;                      // Taille relle du fichier           (*   4b *)
  end;                                                                            (* Total : 140b *)
  PRealFileInfo    = ^TRealFileInfo;

  (* TCompressedFile *)
  TCompressedFile  = packed record
    cfPackedSize   : LongInt;                       // Taille compresse du fichier       (*   4b *)
    cfSerialAlgo   : TInfosSerial;                  // Numero de serie de l'algo          (*  24b *)
    cfCRC32        : LongWord;                      // CRC32 des donnes enregistres     (*   4b *)
    cfErase        : boolean;                       // Vaut 1 quand le fichier est effac (*   1b *)
  end;                                                                            (* Total :  31b *)
  PCompressedFile  = ^TCompressedFile;

  (* TPosition *)
  TPosition        = packed record
    pNumDisk   : Byte;                              // Numro du disque                   (*   1b *)
    pPosInFile : LongWord;                          // Position des donnes sur le disque (*   4b *)
  end;                                                                            (* Total :   5b *)
  PPosition        = ^TPosition;

  (* TFileInformation *)
  TFileInformation = packed record
    fiRealFileInfo : TRealFileInfo;                 // Info. concernant le fichier        (* 140b *)
    fiCompressInfo : TCompressedFile;               // Info. sur le fichier compress     (*  31b *)
    fiBlocAddress  : TPosition;                     // Adresse du fichier dans l'archive  (*   5b *)
  end;                                                                            (* Total : 175b *)
  PFileInformation = ^TFileInformation;

  (* TPutStream *)
  EPutStream = class(Exception);
  TPutStream = class(TStream)
  private
    FPos  : Int64;
    FOnPut: TPutByte;
  protected
    function GetSize: Int64; override;
  public
    constructor Create(put: TPutByte);
    destructor Destroy(); override;
    function Read(var Buffer; Count: Longint): Longint; override;
    function Write(const Buffer; Count: Longint): Longint; override;
    function Seek(const Offset: Int64; Origin: TSeekOrigin): Int64; override;
  end;

  (* TGetStream *)
  EGetStream = class(Exception);
  TGetStream = class(TStream)
  private
    FPos  : Int64;
    FOnGet: TGetByte;
  protected
    function GetSize: Int64; override;
  public
    constructor Create(get: TGetByte);
    function Read(var Buffer; Count: Longint): Longint; override;
    function Write(const Buffer; Count: Longint): Longint; override;
    function Seek(const Offset: Int64; Origin: TSeekOrigin): Int64; override;
  end;

implementation

resourcestring
  rsPutNotImplemented = 'Aucune mthode d''criture dfinie';
  rsGetNotImplemented = 'Aucune mthode de lecture dfinie';
  rsReadError         = 'Lecture dans le flux impossible.';
  rsWriteError        = 'Ecriture dans le flux impossible.';
  rsSeekError         = 'Dplacement impossible';

type
  TBytes = array[0..MaxInt - 1] of Byte;

{ TPutStream }

constructor TPutStream.Create(put: TPutByte);
begin
  if(not Assigned(put)) then
    raise EPutStream.Create(rsPutNotImplemented);
  FOnPut := put;
  FPos := 0;
end;

destructor TPutStream.Destroy;
begin
  FOnPut(0, true);
  inherited;
end;

function TPutStream.GetSize: Int64;
begin
  result := Position;
end;

function TPutStream.Read(var Buffer; Count: Integer): Longint;
begin
  raise EPutStream.Create(rsReadError);
end;

function TPutStream.Seek(const Offset: Int64; Origin: TSeekOrigin): Int64;
begin
  if((Offset = 0) and (Origin = soCurrent)) then
    result := FPos
  else
    raise EPutStream.Create(rsSeekError);
end;

function TPutStream.Write(const Buffer; Count: Integer): Longint;
begin
  result := 0;
  while(result < Count) do begin
    FOnPut(TBytes(Buffer)[Result], false);
    Inc(Result);
  end;
  Inc(FPos, result);
end;

{ TGetStream }

constructor TGetStream.Create(get: TGetByte);
begin
  if(not Assigned(get)) then
    raise EGetStream.Create(rsGetNotImplemented);
  FOnGet := get;
  FPos := 0;
end;

function TGetStream.GetSize: Int64;
begin
  result := Position;
end;

function TGetStream.Read(var Buffer; Count: Integer): Longint;
var C: Byte;
begin
  result := 0;
  while((result < Count) and FOnGet(C)) do begin
    TBytes(Buffer)[result] := C;
    Inc(result);
  end;
end;

function TGetStream.Seek(const Offset: Int64; Origin: TSeekOrigin): Int64;
begin
  if((Offset = 0) and (Origin = soCurrent)) then
    result := FPos
  else
    raise EGetStream.Create(rsSeekError);
end;

function TGetStream.Write(const Buffer; Count: Integer): Longint;
begin
  raise EGetStream.Create(rsWriteError);
end;

end.
