(* GNU-GPL
 * System Compression - Logiciel de Compression/Decompression
 * Copyright (c) 2002 Ulrich Van Den Hekke (ulrich.vdh@free.fr)
 * 24 rue Jean Jaures
 * 59560 Comines
 *
 * Ce programme est libre, vous pouvez le redistribuer et/ou le modifier selon
 * les termes de la Licence Publique Gnrale GNU publie par la
 * Free Software Foundation (version 2 ou bien toute autre version ultrieure
 * choisie par vous).
 *
 * Ce programme est distribu car potentiellement utile, mais
 * SANS AUCUNE GARANTIE, ni explicite ni implicite, y compris les garanties de
 * commercialisation ou d'adaptation dans un but spcifique. Reportez-vous  la
 * Licence Publique Gnrale GNU pour plus de dtails.
 *
 * Vous devez avoir reu une copie de la Licence Publique Gnrale GNU en mme
 * temps que ce programme ; si ce n'est pas le cas, crivez  la
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, tats-Unis.
 *
 *)

(******************************************************************************)
(* Menu Contextuel affich dans l'explorateur Windows                         *)
(* Version : 1.0.0.0                                                          *)
(* Author : Ulrich Van Den Hekke                                              *)
(* Create day : 10/05/2003                                                    *)
(******************************************************************************)

unit ContextMenu;

{$WARN SYMBOL_PLATFORM OFF}

interface

uses
  Windows, ActiveX, ComObj, ShlObj, Dialogs;

type
  TContextMenu = class(TComObject, IShellExtInit, IContextMenu)
  private
    FFileName: array[0..MAX_PATH] of char;
  protected
    { IShellExtInit }
    function IShellExtInit.Initialize = SEIInitialize;
    function SEIInitialize(pidlFolder: PItemIDList; lpdobj: IDataObject;
      hKeyProgID: HKEY): HRESULT; stdcall;
    { IContextMenu }
    function QueryContextMenu(Menu: HMENU; indexMenu: Cardinal;
      idCmdFirst: Cardinal; idCmdLast: Cardinal; uFlags: Cardinal): HRESULT; stdcall;
    function InvokeCommand(var lpici: _CMINVOKECOMMANDINFO): HRESULT; stdcall;
    function GetCommandString(idCmd: Cardinal; uType: Cardinal; pwReserved: PUINT;
      pszName: PAnsiChar; cchMax: Cardinal): HRESULT; stdcall;
  public
  end;

const
  Class_ContextMenu: TGUID = '{5CA1AEB8-3B71-4DE4-BE7A-D76600EB5DBA}';

implementation

uses ComServ, SysUtils, ShellAPI, Registry;

resourcestring
  rsDescription    = 'Menu Contextuel System C';
  rsExecutingError = 'Erreur lors de l''xecution du logiciel';
  rsPathError      = 'Erreur lors du changement du dossier courrant';

{ TContextMenu }

function TContextMenu.GetCommandString(idCmd, uType: Cardinal; pwReserved: PUINT;
  pszName: PAnsiChar; cchMax: Cardinal): HRESULT;
begin
  if(idCmd = 0) then begin
    if(uType = GCS_HELPTEXT) then
      StrCopy(pszName, 'Extraire tous les fichiers dans le rprtoire courant');
    Result := NOERROR;
  end else
    Result := E_INVALIDARG;
end;

function TContextMenu.InvokeCommand(var lpici: _CMINVOKECOMMANDINFO): HRESULT;
var ConsoleSysC, AppPath, PrevPath: string;
    H: THandle;
begin
  Result := E_FAIL;

  if(HiWord(Integer(lpici.lpVerb)) <> 0) then
    Exit;

  if (LoWord(lpici.lpVerb) <> 0) then begin
    Result := E_INVALIDARG;
    Exit;
  end;

  try
    AppPath := GetRegStringValue('Software\Url''tils\System C\1.0.0.0', 'AppPath', HKEY_CURRENT_USER);
    ConsoleSysC := Format('%s\syscons.exe -e "%s"', [ExtractShortPathName(AppPath), FFileName]);

    PrevPath := GetCurrentDir;
    try
      if(not SetCurrentDir(ExtractFilePath(FFileName))) then
        raise Exception.CreateRes(@rsPathError);
      H := WinExec(PChar(ConsoleSysC), lpici.nShow);
      if(H < 32) then
        raise Exception.Create(rsExecutingError + '''' + ConsoleSysC + '''');
      Result := NOERROR;
    finally
      SetCurrentDir(PrevPath);
    end;
  except
    on E: Exception do
      MessageBox(lpici.hwnd, PChar(E.Message), 'Erreur', MB_OK or MB_ICONSTOP);
  end;
end;

function TContextMenu.QueryContextMenu(Menu: HMENU; indexMenu, idCmdFirst, idCmdLast,
  uFlags: Cardinal): HRESULT;
begin
  result := MakeResult(SEVERITY_SUCCESS, FACILITY_NULL, 0);
  if((uFlags and $0000000F) = CMF_NORMAL) or ((uFlags and CMF_EXPLORE) <> 0) then begin
    InsertMenu(Menu, indexMenu, MF_STRING or MF_BYPOSITION, idCmdFirst, '&System C -> Extract');
    result := MakeResult(SEVERITY_SUCCESS, FACILITY_NULL, 1);
  end;
end;

function TContextMenu.SEIInitialize(pidlFolder: PItemIDList; lpdobj: IDataObject;
  hKeyProgID: HKEY): HRESULT;
var StgMedium: TStgMedium;
    FormatEtc: TFormatEtc;
begin
  if(lpdobj = nil) then begin
    Result := E_INVALIDARG;
    Exit;
  end;

  with FormatEtc do begin
    cfFormat := CF_HDROP;
    ptd      := nil;
    dwAspect := DVASPECT_CONTENT;
    lindex   := -1;
    tymed    := TYMED_HGLOBAL;
  end;

  result := lpdobj.GetData(FormatEtc, StgMedium);
  if(Failed(result)) then
    Exit;

  if(DragQueryFile(StgMedium.hGlobal, $FFFFFFFF, nil, 0) = 1) then begin
    DragQueryFile(StgMedium.hGlobal,0, FFileName, SizeOf(FFileName));
    Result := NOERROR;
  end else begin
    FFileName[0] := #0;
    Result := E_FAIL;
  end;
  ReleaseStgMedium(StgMedium);
end;

type
  TContextMenuFactory = class(TComObjectFactory)
  public
    procedure UpdateRegistry(register: boolean); override;
  end;

{ TContextMenuFactory }

procedure TContextMenuFactory.UpdateRegistry(register: boolean);
var
  ClassID: string;
begin
  if(register) then begin
    inherited;

    ClassID := GUIDToString(Class_ContextMenu);
    CreateRegKey('System C Archive\shellex', '', '');
    CreateRegKey('System C Archive\shellex\ContextMenuHandlers', '', '');
    CreateRegKey('System C Archive\shellex\ContextMenuHandlers\ContMenu', '', ClassID);

    if(Win32Platform = VER_PLATFORM_WIN32_NT) then begin
      with TRegistry.Create do
      try
        RootKey := HKEY_LOCAL_MACHINE;
        OpenKey('Software\Microsoft\CurrentVersion\Shell Extensions', True);
        OpenKey('Approved', True);
        WriteString(ClassID, rsDescription);
      finally
        Free;
      end;
    end;
  end else begin
    DeleteRegKey('System C Archive\shellex\ContextMenuHandlers\ContMenu');
    DeleteRegKey('System C Archive\shellex\ContextMenuHandlers');
    DeleteRegKey('System C Archive\shellex');

    inherited ;
  end;
end;

initialization
  TContextMenuFactory.Create(ComServer, TContextMenu, Class_ContextMenu,
    'ContextMenu', rsDescription, ciMultiInstance, tmApartment);
end.
