(* GNU-GPL
 * System Compression - Logiciel de Compression/Decompression
 * Copyright (c) 2002 Ulrich Van Den Hekke (ulrich.vdh@free.fr)
 * 24 rue Jean Jaures
 * 59560 Comines
 *
 * Ce programme est libre, vous pouvez le redistribuer et/ou le modifier selon
 * les termes de la Licence Publique Gnrale GNU publie par la
 * Free Software Foundation (version 2 ou bien toute autre version ultrieure
 * choisie par vous).
 *
 * Ce programme est distribu car potentiellement utile, mais
 * SANS AUCUNE GARANTIE, ni explicite ni implicite, y compris les garanties de
 * commercialisation ou d'adaptation dans un but spcifique. Reportez-vous  la
 * Licence Publique Gnrale GNU pour plus de dtails.
 *
 * Vous devez avoir reu une copie de la Licence Publique Gnrale GNU en mme
 * temps que ce programme ; si ce n'est pas le cas, crivez  la
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, tats-Unis.
 *
 *)

(******************************************************************************)
(* Informations enregistres dans la base de registre                         *)
(* Version : 1.0.0.0                                                          *)
(* Author : Ulrich Van Den Hekke                                              *)
(* Create day : 06/07/2002                                                    *)
(******************************************************************************)

unit reginfos;

interface

uses Windows, SysUtils, Classes, BaseRegistry;

type
  TRegInfos = class(TBaseRegistry)
  private
    FPluginsDir: string;
    FWizardPos: TPoint;
    FMRUList: TStrings;
    FAppPath: string;
    function GetWizardPos: PPoint;
  protected
  public
    constructor Create();
    destructor Destroy(); override;

    procedure Default; override;
    function SaveToRegistry: boolean; override;
    procedure LoadFromRegistry; override;

    property PluginsDir: string read FPluginsDir;
    property AppPath: string read FAppPath;
    property WizardPos: PPoint read GetWizardPos;
    property MRUList: TStrings read FMRUList;
  end;

var
  RegInfosOption: TRegInfos;

implementation

uses Math;

resourcestring
  rsPluginsDir= 'Plugins Dir';
  rsAppPath   = 'AppPath';
  rsWizardPos = '\Wizard Pos';
  rsPosX      = 'Pos X';
  rsPosY      = 'Pos Y';
  rsMRUList   = '\MRU File List';

const
  dfPosX      = -1;
  dfPosY      = -1;
  dfMRUCount  = 10;

{ TRegInfos }

constructor TRegInfos.Create;
begin
  inherited Create(rddUser);
  FMRUList := TStringList.Create();
end;

procedure TRegInfos.Default;
begin
  if(Assigned(FMRUList)) then FMRUList.Clear();
  FWizardPos := Point(dfPosX, dfPosY);
end;

destructor TRegInfos.Destroy;
begin
  FMRUList.Free();
  inherited Destroy();
end;

function TRegInfos.GetWizardPos: PPoint;
begin
  result := @FWizardPos;
end;

procedure TRegInfos.LoadFromRegistry;
var I  : Integer;
    MRU: string;
begin
  inherited LoadFromRegistry();

  if(Registry.OpenKey(getVersionKey, false)) then
  try
    FPluginsDir := ReadValue(rsPluginsDir, ExtractFilePath(ParamStr(0)));
    FAppPath    := ReadValue(rsAppPath,    ExtractFilePath(ParamStr(0)));
  finally
    Registry.CloseKey;
  end;

  if(Registry.OpenKey(getVersionKey + rsWizardPos, false)) then
  try
    FWizardPos.x := ReadValue(rsPosX, dfPosX);
    FWizardPos.y := ReadValue(rsPosY, dfPosY);
  finally
    Registry.CloseKey;
  end;

  if(Registry.OpenKey(getVersionKey + rsMRUList, false)) then
  try
    FMRUList.Clear();

    for i := 1 to dfMRUCount do begin
      MRU := ReadValue(IntToStr(I), EmptyStr);
      if(MRU <> EmptyStr) then
        FMRUList.Insert(0, MRU);
    end;
  finally
    Registry.CloseKey;
  end;
end;

function TRegInfos.SaveToRegistry: boolean;
var i: integer;
begin
  result := inherited SaveToRegistry();

  if(Registry.OpenKey(getVersionKey + rsWizardPos, true)) then
  try
    result := WriteValue(rsPosX, FWizardPos.x, dfPosX) or result;
    result := WriteValue(rsPosY, FWizardPos.y, dfPosY) or result;
  finally
    Registry.CloseKey;
  end;

  if(Registry.OpenKey(getVersionKey + rsMRUList, true)) then
  try
    try
      for i := Min(dfMRUCount, FMRUList.Count) downto 1 do
        WriteValue(IntToStr(I), FMRUList[FMRUList.Count - I], EmptyStr);
    except
      on E: EStringListError do ;
      else raise;
    end;
  finally
    Registry.CloseKey;
  end;
end;

initialization
  RegInfosOption := TRegInfos.Create();
  RegInfosOption.LoadFromRegistry();

finalization
  RegInfosOption.SaveToRegistry();
  RegInfosOption.Free();

end.
