(* GNU-GPL
 * System Compression - Logiciel de Compression/Decompression
 * Copyright (c) 2002 Ulrich Van Den Hekke (ulrich.vdh@free.fr)
 * 24 rue Jean Jaures
 * 59560 Comines
 *
 * Ce programme est libre, vous pouvez le redistribuer et/ou le modifier selon
 * les termes de la Licence Publique Gnrale GNU publie par la
 * Free Software Foundation (version 2 ou bien toute autre version ultrieure
 * choisie par vous).
 *
 * Ce programme est distribu car potentiellement utile, mais
 * SANS AUCUNE GARANTIE, ni explicite ni implicite, y compris les garanties de
 * commercialisation ou d'adaptation dans un but spcifique. Reportez-vous  la
 * Licence Publique Gnrale GNU pour plus de dtails.
 *
 * Vous devez avoir reu une copie de la Licence Publique Gnrale GNU en mme
 * temps que ce programme ; si ce n'est pas le cas, crivez  la
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, tats-Unis.
 *
 *)

unit scAddForm;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, BrowseDr, syswizconst, AlgoList;

type
  TAddForm = class(TForm)
    GroupBoxFile: TGroupBox;
    GroupBoxDir: TGroupBox;
    GroupBoxWebAddr: TGroupBox;
    LabelFile: TLabel;
    EditFileName: TEdit;
    ButtonBrowns1: TButton;
    LabelDir: TLabel;
    EditDirName: TEdit;
    ButtonBrown2: TButton;
    CheckBoxRecurse: TCheckBox;
    LabelAddr: TLabel;
    EditWebAddr: TEdit;
    GroupBoxProxy: TGroupBox;
    LabelServeur: TLabel;
    LabelPort: TLabel;
    EditServeur: TEdit;
    EditPort: TEdit;
    ButtonCancel: TButton;
    ButtonOk: TButton;
    CheckBoxProxy: TCheckBox;
    RadioButtonFile: TRadioButton;
    RadioButtonDir: TRadioButton;
    RadioButtonWeb: TRadioButton;
    AddFileDialog: TOpenDialog;
    AddDirDialog: TBrowseDirectoryDlg;
    GroupBoxPlug: TGroupBox;
    CheckBoxAutoPlug: TCheckBox;
    LabelPlug: TLabel;
    ComboBoxPlugins: TComboBox;
    ButtonAboutPlug: TButton;
    procedure StatChange(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure ButtonBrowns1Click(Sender: TObject);
    procedure ButtonBrown2Click(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure ButtonAboutPlugClick(Sender: TObject);
    procedure EditPortKeyPress(Sender: TObject; var Key: Char);
  private
    FPlgList: TAlgoPluginsList;
    function GetAddingType: TAddingType;
    { Dclarations prives }
  public
    { Dclarations publiques }
    property PlgList: TAlgoPluginsList read FPlgList write FPlgList;
    property Position: TAddingType read GetAddingType;
  end;

function GetAddFile(List: TAlgoPluginsList): TList;

implementation

uses Clipbrd, FormsUtils;

{$R *.DFM}

function GetAddFile(List: TAlgoPluginsList): TList;
var FileInfo: PAddingFile;
    ListFile: TStrings;
    I       : Integer;
begin
  Result := TList.Create();
  with(TAddForm.Create(Application.MainForm)) do try
    PlgList  := List;
    ListFile := nil;
    if(ShowModal = mrOk) then try
      ListFile := TStringList.Create();
      case(Position) of
        atFile     : ListFile.CommaText := EditFileName.Text;
        atDirectory: ListFile.CommaText := EditDirName.Text;
        atWebAddr  : ListFile.CommaText := EditWebAddr.Text;
      end;
      for i := 0 to ListFile.Count - 1 do begin
        GetMem(FileInfo, sizeof(FileInfo^));
        FillChar(FileInfo^, sizeof(FileInfo^), #0);
        FileInfo.afMode := Position;
        case(FileInfo.afMode) of
          atFile     : begin
            GetMem(FileInfo.afFileName, length(ListFile[I]) + 1);
            StrPCopy(FileInfo.afFileName, ListFile[I]);
          end;
          atDirectory: begin
            GetMem(FileInfo.afDirName, length(ListFile[I]) + 1);
            StrPCopy(FileInfo.afDirName, ListFile[I]);
            FileInfo.afRecurse := CheckBoxRecurse.Checked;
          end;
          atWebAddr  : begin
            GetMem(FileInfo.afAddrName, length(ListFile[I]) + 1);
            StrPCopy(FileInfo.afAddrName, ListFile[I]);
            if(CheckBoxProxy.Checked) then begin
              GetMem(FileInfo.afProxyName, length(EditServeur.Text) + 1);
              StrPCopy(FileInfo.afProxyName, EditServeur.Text);
              FileInfo.afProxyPort := StrToInt(EditPort.Text);
            end else begin
              FileInfo.afProxyName := nil;
              FileInfo.afProxyPort := 0;
            end;
          end;
        end;
        if(not CheckBoxAutoPlug.Checked) then
          FileInfo.afAlgoIndex := ComboBoxPlugins.ItemIndex
        else
          FileInfo.afAlgoIndex := -1;
        Result.Add(FileInfo);
      end;
    finally
      ListFile.Free;
    end;
  finally
    Free;
  end;
  Result.Pack();
end;

procedure TAddForm.StatChange(Sender: TObject);
begin
  GroupBoxFile.Enabled    := RadioButtonFile.Checked;
  LabelFile.Enabled       := GroupBoxFile.Enabled;
  EditFileName.Enabled    := GroupBoxFile.Enabled;
  ButtonBrowns1.Enabled   := GroupBoxFile.Enabled;

  GroupBoxDir.Enabled     := RadioButtonDir.Checked;
  LabelDir.Enabled        := GroupBoxDir.Enabled;
  EditDirName.Enabled     := GroupBoxDir.Enabled;
  ButtonBrown2.Enabled    := GroupBoxDir.Enabled;
  CheckBoxRecurse.Enabled := GroupBoxDir.Enabled;

  GroupBoxWebAddr.Enabled := RadioButtonWeb.Checked;
  LabelAddr.Enabled       := GroupBoxWebAddr.Enabled;
  EditWebAddr.Enabled     := GroupBoxWebAddr.Enabled;
  CheckBoxProxy.Enabled   := GroupBoxWebAddr.Enabled;
  GroupBoxProxy.Enabled   := GroupBoxWebAddr.Enabled and CheckBoxProxy.Checked;
  LabelServeur.Enabled    := GroupBoxProxy.Enabled;
  EditServeur.Enabled     := GroupBoxProxy.Enabled;
  LabelPort.Enabled       := GroupBoxProxy.Enabled;
  EditPort.Enabled        := GroupBoxProxy.Enabled;

  GroupBoxPlug.Enabled    := not CheckBoxAutoPlug.Checked;
  LabelPlug.Enabled       := GroupBoxPlug.Enabled;
  ComboBoxPlugins.Enabled := GroupBoxPlug.Enabled;
  ButtonAboutPlug.Enabled := GroupBoxPlug.Enabled;

  ButtonOk.Enabled        := (GroupBoxFile.Enabled    and (EditFileName.Text <> '')) or
                             (GroupBoxDir.Enabled     and (EditDirName.Text  <> '')) or
                             (GroupBoxWebAddr.Enabled and (EditWebAddr.Text  <> ''));
end;

procedure TAddForm.FormCreate(Sender: TObject);
begin
  CenterForm(Self);
  RadioButtonFile.Checked  := true;
  CheckBoxAutoPlug.Checked := true;
  if(Clipboard.HasFormat(CF_TEXT) and ((pos('http://', Trim(Clipboard.AsText)) = 1) or (pos('ftp://', Trim(Clipboard.AsText)) = 1))) then
    EditWebAddr.Text := Clipboard.AsText;
end;

procedure TAddForm.ButtonBrowns1Click(Sender: TObject);
begin
  if(AddFileDialog.Execute) then begin
    EditFileName.Text := AddFileDialog.Files.CommaText;
  end;
end;

procedure TAddForm.ButtonBrown2Click(Sender: TObject);
begin
  if(AddDirDialog.Execute) then begin
    EditDirName.Text := '"' + AddDirDialog.Selection + '"';
  end;
end;

procedure TAddForm.FormShow(Sender: TObject);
var i: integer;
begin
  for I := 0 to PlgList.Count - 1 do
    ComboBoxPlugins.Items.Add(Trim(PlgList[I].PluginsInformation.iName));
  ComboBoxPlugins.ItemIndex := 0;
end;

procedure TAddForm.ButtonAboutPlugClick(Sender: TObject);
begin
  PlgList[ComboBoxPlugins.ItemIndex].About(Handle);
end;

function TAddForm.GetAddingType: TAddingType;
begin
  if(RadioButtonFile.Checked) then result := atFile else
  if(RadioButtonDir .Checked) then result := atDirectory else
  if(RadioButtonWeb .Checked) then result := atWebAddr else result := atFile;
end;

procedure TAddForm.EditPortKeyPress(Sender: TObject; var Key: Char);
begin
  if(not (Key in ['0'..'9'])) then Key := #0; 
end;

end.
