(* GNU-GPL
 * System Compression - Logiciel de Compression/Decompression
 * Copyright (c) 2002 Ulrich Van Den Hekke (ulrich.vdh@free.fr)
 * 24 rue Jean Jaures
 * 59560 Comines
 *
 * Ce programme est libre, vous pouvez le redistribuer et/ou le modifier selon
 * les termes de la Licence Publique Gnrale GNU publie par la
 * Free Software Foundation (version 2 ou bien toute autre version ultrieure
 * choisie par vous).
 *
 * Ce programme est distribu car potentiellement utile, mais
 * SANS AUCUNE GARANTIE, ni explicite ni implicite, y compris les garanties de
 * commercialisation ou d'adaptation dans un but spcifique. Reportez-vous  la
 * Licence Publique Gnrale GNU pour plus de dtails.
 *
 * Vous devez avoir reu une copie de la Licence Publique Gnrale GNU en mme
 * temps que ce programme ; si ce n'est pas le cas, crivez  la
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, tats-Unis.
 *
 *)

(******************************************************************************)
(* Fonction utilitaire du logiciel en mode console                            *)
(* Version : 1.0.0.0                                                          *)
(* Author : Ulrich Van Den Hekke                                              *)
(* Create day : 06/07/2002                                                    *)
(******************************************************************************)

unit sysconsutils;

interface

uses Windows, Classes, SysUtils, AlgoList, scFileStruct;

procedure showHelp(const Text : string = 'HELP');
procedure showCredits();
procedure showLicence();
procedure showPluginsList(List: TAlgoPluginsList);
procedure showFileList(Archive: TSystemCFileStruct; const filename: string = '');
procedure AddFiles(Archive: TSystemCFileStruct; Parent, CurrPath, CurrFile, Algo: string; Recurse: boolean);
procedure ExtractFile(Archive: TSystemCFileStruct; DestDir, FileName: string; FileNum: integer = 1);
procedure DeleteFile(var Archive: TSystemCFileStruct; FileName: string = ''; FileNum: integer = 1);
procedure TestAlgo(List: TAlgoPluginsList; Algo_Name: String; showProgress: TOnProgression);
procedure InitialiseArchive(var ArchiveName: TFileName; var Archive: TFileStream);
procedure FinalliseArchive(var Archive: TFileStream);

implementation

uses FileTools, sysconsconst, syscconst;

procedure showHelp(const Text : string);
var List: TStringList;
    St  : TStream;
    I   : Integer;
    Ch  : PChar;
begin
  List := nil; St := nil;
  try
    List := TStringList.Create;
    St   := TResourceStream.Create(HInstance, Pchar(Text), RT_RCDATA);

    List.LoadFromStream(St);

    for i := 0 to List.Count - 1 do begin
      GetMem(Ch, Length(List[I]) + 1);
      try
        CharToOem(PChar(List[I]), Ch);
        WriteLn(Ch);
      finally
        FreeMem(Ch);
      end;
    end;
  finally
    St.Free;
    List.Free;
  end;
  raise EAbort.Create(rsHaltApp);
end;

procedure showCredits();
begin
  showHelp('CREDITS');
end;

procedure showLicence();
begin
  showHelp('LICENCE');
end;

procedure showPluginsList(List: TAlgoPluginsList);
var I: Integer;
begin
  WriteLn(rsTitleList);
  for I := 0 to List.Count - 1 do
  begin
    WriteLn(Format(rsTextFormList, [PChar(String(List.Items[I].PluginsInformation.iName)), PChar(String(List.Items[I].PluginsInformation.iDescription))]));
  end;
  raise EAbort.Create(rsHaltApp);
end;

procedure showFileList(Archive: TSystemCFileStruct; const filename: string);
var Index: Integer;
    S    : string;
begin
  WriteLn(rsListCol);
  WriteLn(rsListSep);
  for index := 0 to Archive.Count - 1 do begin
    if((not Archive[Index].fiCompressInfo.cfErase) and ((filename = EmptyStr) or (UpperCase(Trim(Archive[Index].fiRealFileInfo.rfiName)) = UpperCase(filename)))) then begin
      try
        S := Trim(Archive.AlgoList.Items[Archive.AlgoList.IndexOf(Archive[Index].fiCompressInfo.cfSerialAlgo)].PluginsInformation.iName);
      except
        S := rsUnknown;
      end;
      S := Format(rsListLine, [
                    Trim(Archive[Index].fiRealFileInfo.rfiName),
                    SizeToStr(Archive[Index].fiRealFileInfo.rfiSize, bytesuntis),
                    DateToStr(Archive[Index].fiRealFileInfo.rfiModifiedDate),
                    IntToHex(Archive[Index].fiCompressInfo.cfCRC32, 8),
                    S,
                    SizeToStr(Archive[Index].fiCompressInfo.cfPackedSize, bytesuntis)
                  ]);
      Write(S);

      WriteLn;
    end;
  end;
  raise EAbort.Create(rsHaltApp);
end;

procedure AddFiles(Archive: TSystemCFileStruct; Parent, CurrPath, CurrFile, Algo: string; Recurse: boolean);
var SRec      : TSearchRec;
    Index, res: integer;
    Car       : Char;
begin
  FillChar(SRec, sizeof(SRec), #0);

  res  := FindFirst(CurrPath + CurrFile, faAnyFile, SRec);
  if(res <> 0) then WriteLn(Format(rsNoFileIn, [CurrPath + CurrFile]));
  while(res = 0) do begin
    if(not ((SRec.Attr and faVolumeID > 0) or (SRec.Attr and faDirectory > 0))) then
    try
      if(UpperCase(Algo) = cstAutoPlugin) then Index := -1 else Index := Archive.AlgoList.IndexOf(Algo);

      Index := Archive.Add(CurrPath + SRec.Name, Index, parent);
      WriteLn(Format(rsAddFinish, [ExtractFileName(ExtractShortPathName(CurrPath + SRec.Name)),
                                   SizeToStr(Archive[Index].fiRealFileInfo.rfiSize, bytesuntis),
                                   SizeToStr(Archive[Index].fiCompressInfo.cfPackedSize, bytesuntis)]));
    except
      on E: Exception do begin
        WriteLn(E.Message);
        repeat
          WriteLn(rsIgnoreCancel); ReadLn(Car);
          case(Car)of
            'I', 'i': break;
            'C', 'c',
            'A', 'a': raise EAbort.Create(E.Message);
          end
        until true;
      end;
    end;
    res := FindNext(SRec);
  end;
  FindClose(SRec);

  if(recurse) then begin
    res  := FindFirst(CurrPath + '*.*', faAnyFile, SRec);
    while(res = 0) do begin
      if((SRec.Attr and faDirectory > 0) and (SRec.Name <> '.') and (SRec.Name <> '..')) then
        AddFiles(Archive, Parent, CurrPath + SRec.Name + '\', CurrFile, Algo, Recurse);
      res := FindNext(SRec);
    end;
    FindClose(SRec);
  end;
end;

procedure ExtractFile(Archive: TSystemCFileStruct; DestDir, FileName: string; FileNum: integer);
var Index: Integer;
begin
  if(FileName <> '') then begin
    Index := Archive.IndexOf(FileName, FileNum);
    if(Index < 0) then WriteLn(Format(rsFileNotFound, [FileName, FileNum])) else begin
      Archive.Extract(index, DestDir);
      WriteLn(Format(rsExtractFinish, [Trim(Archive[Index].fiRealFileInfo.rfiName),
                                       SizeToStr(Archive[Index].fiCompressInfo.cfPackedSize, bytesuntis),
                                       SizeToStr(Archive[Index].fiRealFileInfo.rfiSize, bytesuntis)]));
    end;
  end else begin
    for Index := 0 to Archive.Count - 1 do begin
      Archive.Extract(index, DestDir);
      WriteLn(Format(rsExtractFinish, [Trim(Archive[Index].fiRealFileInfo.rfiName),
                                       SizeToStr(Archive[Index].fiCompressInfo.cfPackedSize, bytesuntis),
                                       SizeToStr(Archive[Index].fiRealFileInfo.rfiSize, bytesuntis)]));
    end;
  end;
end;

procedure DeleteFile(var Archive: TSystemCFileStruct; FileName: string = ''; FileNum: integer = 1);
var Index: Integer;
begin
  if(FileName = EmptyStr) then begin
    Archive.Free;
    Archive := nil;
  end else begin
    Index := Archive.IndexOf(FileName, FileNum);
    if(Index < 0) then WriteLn(Format(rsFileNotFound, [FileName, FileNum])) else begin
      Archive.Delete(index);
      WriteLn(Format(rsDeleteFinish, [FileName]));
    end;
  end;
end;

procedure TestAlgo(List: TAlgoPluginsList; Algo_Name: String; showProgress: TOnProgression);
var Index        : Integer;
    TestPlug     : TAlgoPlugins;
    St1, St2, St3: TStream;
begin
  try
    Index    := List.IndexOf(Algo_Name);
    TestPlug := List.Items[Index];
  except
    on E: EListError do begin
      WriteLn(Format(rsBadPlugins, [Algo_Name]));
      raise EAbort.Create(rsHaltApp);
    end else
      raise;
  end;

  TestPlug.OnProgression := showProgress;
  WriteLn(Format(rsTestLine, [PChar(String(TestPlug.PluginsInformation.iName))]) + #10);

  St1 := nil;
  St2 := nil;
  St3 := nil;
  try
    St1 := TestPlug.RandomStreamTest(cstTestBuffer);
    WriteLn(Format(rsData2Compress, [St1.Size]));
    St2 := TestPlug.PackStreamTest(St1);
    WriteLn(Format(rsData2Decomp, [St2.Size]));
    St3 := TestPlug.UnpackStreamTest(St1.Size, St2);
    WriteLn(Format(rsDataFinal, [St3.Size]));
    if(TestPlug.CompareStreamTest(St1, St3)) then
      WriteLn(rsDataEqual)
    else begin
      WriteLn(rsDataDiff);
      List.RemovePlugins(Index);
    end;
  finally
    St3.Free;
    St2.Free;
    St1.Free;
  end;
end;

procedure InitialiseArchive(var ArchiveName: TFileName; var Archive: TFileStream);
begin
  ArchiveName := ExpandFileName(ArchiveName);
  if(ExtractFileExt(ArchiveName) = '') then ArchiveName := ArchiveName + '.SC2';

  if(FileExists(ArchiveName)) then
    Archive := TFileStream.Create(ArchiveName, fmOpenReadWrite or fmShareDenyWrite)
  else
    Archive := TFileStream.Create(ArchiveName, fmCreate or fmShareDenyWrite);
end;

procedure FinalliseArchive(var Archive: TFileStream);
begin
  Archive.Free;
  Archive := nil;
  Writeln(rsEndFlush);
end;

end.
